@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2014 Jeffrey Stedfast
+ *  Copyright (C) 2000-2017 Jeffrey Stedfast
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public License
@@ -149,8 +149,8 @@ int main (int argc, char **argv)
 	header_decode_init ();
 	
 	printf ("/* THIS FILE IS AUTOGENERATED: DO NOT EDIT! */\n\n");
-	printf ("/**\n * To regenerate:\n * make gen-table\n");
-	printf (" * ./gen-table > gmime-table-private.h\n **/\n\n");
+	printf ("/*\n * To regenerate:\n * make gen-table\n");
+	printf (" * ./gen-table > gmime-table-private.h\n */\n\n");
 	
 	/* print out the table */
 	printf ("static unsigned short gmime_special_table[256] = {");
@@ -206,7 +206,7 @@ int main (int argc, char **argv)
 	printf ("#define CHARS_ATTRCHAR \"*'%% \"              /* attribute-char from rfc2184 */\n");
 	printf ("\n");
 	
-	printf ("#define GMIME_FOLD_LEN 76\n");
+	printf ("#define GMIME_FOLD_LEN 78\n");
 	
 	return 0;
 }
