@@ -123,6 +123,7 @@ g_mime_autocrypt_header_new (void)
 
 /**
  * g_mime_autocrypt_header_new_from_string:
+ * @string: The raw string value of an Autocrypt header
  *
  * Creates a new #GMimeAutocryptHeader object based on the value of an
  * Autocrypt: header.
@@ -139,9 +140,10 @@ GMimeAutocryptHeader *
 g_mime_autocrypt_header_new_from_string (const char *string)
 {
 	GMimeAutocryptHeader *ret = NULL;
+	GBytes *newkeydata = NULL;
 	gchar **ksplit = NULL;
 	gchar *kjoined = NULL;
-	GBytes *newkeydata = NULL;
+	
 	if (string == NULL)
 		return NULL;
 	
@@ -149,7 +151,7 @@ g_mime_autocrypt_header_new_from_string (const char *string)
 	   both of which might happen to mails in transit. So this
 	   could be improved.  It's also probably not currently the
 	   most efficient implementation. */
-
+	
 	struct _attr { char *val; const char *name; size_t sz; int count; };
 #define attr(n, str) struct _attr n = { .name = str, .sz = sizeof(str)-1 }
 	attr(k, "keydata");
@@ -182,7 +184,7 @@ g_mime_autocrypt_header_new_from_string (const char *string)
 			
 		}
 	}
-
+	
 	if (k.count != 1)
 		goto done;
 	if (a.count != 1)
@@ -190,7 +192,7 @@ g_mime_autocrypt_header_new_from_string (const char *string)
 	if (p.count > 1)
 		goto done;
 	GMimeAutocryptPreferEncrypt newpref = GMIME_AUTOCRYPT_PREFER_ENCRYPT_NONE;
-
+	
 	if (p.count && (g_ascii_strcasecmp ("mutual", p.val) == 0))
 		newpref = GMIME_AUTOCRYPT_PREFER_ENCRYPT_MUTUAL;
 	
@@ -206,8 +208,7 @@ g_mime_autocrypt_header_new_from_string (const char *string)
 	g_mime_autocrypt_header_set_keydata (ret, newkeydata);
 	
  done:
-	if (vals)
-		g_strfreev (vals);
+	g_strfreev (vals);
 	if (ksplit)
 		g_strfreev (ksplit);
 	if (newkeydata)
@@ -231,9 +232,11 @@ g_mime_autocrypt_header_set_address_from_string (GMimeAutocryptHeader *ah, const
 
 	if (ah->address)
 		g_object_unref (ah->address);
-	ah->address = INTERNET_ADDRESS_MAILBOX(internet_address_mailbox_new (NULL, address));
+	
+	ah->address = (InternetAddressMailbox *) internet_address_mailbox_new (NULL, address);
 }
 
+
 /**
  * g_mime_autocrypt_header_set_address:
  * @ah: a #GMimeAutocryptHeader object
@@ -245,14 +248,15 @@ void
 g_mime_autocrypt_header_set_address (GMimeAutocryptHeader *ah, InternetAddressMailbox *address)
 {
 	g_return_if_fail (GMIME_IS_AUTOCRYPT_HEADER (ah));
-
+	g_return_if_fail (INTERNET_ADDRESS_IS_MAILBOX (address));
+	
+	g_object_ref (address);
 	if (ah->address)
 		g_object_unref (ah->address);
-	
 	ah->address = address;
-	g_object_ref (address);
 }
 
+
 /**
  * g_mime_autocrypt_header_get_address:
  * @ah: a #GMimeAutocryptHeader object
@@ -261,11 +265,11 @@ g_mime_autocrypt_header_set_address (GMimeAutocryptHeader *ah, InternetAddressMa
  *
  * Returns: (transfer none): the address associated with the Autocrypt header
  **/
-InternetAddressMailbox*
+InternetAddressMailbox *
 g_mime_autocrypt_header_get_address (GMimeAutocryptHeader *ah)
 {
 	g_return_val_if_fail (GMIME_IS_AUTOCRYPT_HEADER (ah), NULL);
-
+	
 	return ah->address;
 }
 
@@ -278,11 +282,11 @@ g_mime_autocrypt_header_get_address (GMimeAutocryptHeader *ah)
  *
  * Returns: (transfer none): the address associated with the Autocrypt header
  **/
-const char*
+const char *
 g_mime_autocrypt_header_get_address_as_string (GMimeAutocryptHeader *ah)
 {
 	g_return_val_if_fail (GMIME_IS_AUTOCRYPT_HEADER (ah), NULL);
-
+	
 	return ah->address->addr;
 }
 
@@ -298,10 +302,11 @@ void
 g_mime_autocrypt_header_set_prefer_encrypt (GMimeAutocryptHeader *ah, GMimeAutocryptPreferEncrypt pref)
 {
 	g_return_if_fail (GMIME_IS_AUTOCRYPT_HEADER (ah));
-
+	
 	ah->prefer_encrypt = pref;
 }
 
+
 /**
  * g_mime_autocrypt_header_get_prefer_encrypt:
  * @ah: a #GMimeAutocryptHeader object
@@ -314,7 +319,7 @@ GMimeAutocryptPreferEncrypt
 g_mime_autocrypt_header_get_prefer_encrypt (GMimeAutocryptHeader *ah)
 {
 	g_return_val_if_fail (GMIME_IS_AUTOCRYPT_HEADER (ah), GMIME_AUTOCRYPT_PREFER_ENCRYPT_NONE);
-
+	
 	return ah->prefer_encrypt;
 }
 
@@ -330,15 +335,17 @@ void
 g_mime_autocrypt_header_set_keydata (GMimeAutocryptHeader *ah, GBytes *keydata)
 {
 	g_return_if_fail (GMIME_IS_AUTOCRYPT_HEADER (ah));
-
+	
+	if (keydata)
+		g_bytes_ref (keydata);
+	
 	if (ah->keydata)
 		g_bytes_unref (ah->keydata);
 	
 	ah->keydata = keydata;
-	if (keydata)
-		g_bytes_ref (keydata);
 }
 
+
 /**
  * g_mime_autocrypt_header_get_keydata:
  * @ah: a #GMimeAutocryptHeader object
@@ -351,7 +358,7 @@ GBytes*
 g_mime_autocrypt_header_get_keydata (GMimeAutocryptHeader *ah)
 {
 	g_return_val_if_fail (GMIME_IS_AUTOCRYPT_HEADER (ah), NULL);
-
+	
 	return ah->keydata;
 }
 
@@ -367,15 +374,17 @@ void
 g_mime_autocrypt_header_set_effective_date (GMimeAutocryptHeader *ah, GDateTime *effective_date)
 {
 	g_return_if_fail (GMIME_IS_AUTOCRYPT_HEADER (ah));
-
+	
+	if (effective_date)
+		g_date_time_ref (effective_date);
+	
 	if (ah->effective_date)
 		g_date_time_unref (ah->effective_date);
 	
 	ah->effective_date = effective_date;
-	if (effective_date)
-		g_date_time_ref (effective_date);
 }
 
+
 /**
  * g_mime_autocrypt_header_get_effective_date:
  * @ah: a #GMimeAutocryptHeader object
@@ -384,14 +393,15 @@ g_mime_autocrypt_header_set_effective_date (GMimeAutocryptHeader *ah, GDateTime
  *
  * Returns: (transfer none): the effective date associated with the Autocrypt header
  **/
-GDateTime*
+GDateTime *
 g_mime_autocrypt_header_get_effective_date (GMimeAutocryptHeader *ah)
 {
 	g_return_val_if_fail (GMIME_IS_AUTOCRYPT_HEADER (ah), NULL);
-
+	
 	return ah->effective_date;
 }
 
+
 /**
  * g_mime_autocrypt_header_is_complete:
  * @ah: a #GMimeAutocryptHeader object
@@ -408,6 +418,7 @@ gboolean
 g_mime_autocrypt_header_is_complete (GMimeAutocryptHeader *ah)
 {
 	g_return_val_if_fail (GMIME_IS_AUTOCRYPT_HEADER (ah), FALSE);
+	
 	return (ah->address && ah->address->addr && ah->keydata &&
 		g_bytes_get_size (ah->keydata) && g_bytes_get_data (ah->keydata, NULL));
 }
@@ -441,9 +452,7 @@ g_mime_autocrypt_header_to_string (GMimeAutocryptHeader *ah, gboolean gossip)
 		pe = "prefer-encrypt=mutual; ";
 	const char *addr = internet_address_mailbox_get_addr (ah->address);
 	GPtrArray *lines = g_ptr_array_new_with_free_func (g_free);
-
-	gchar * first = g_strdup_printf("addr=%s; %skeydata=",
-					addr, pe);
+	gchar *first = g_strdup_printf ("addr=%s; %skeydata=", addr, pe);
 	size_t n = strlen (first);
 	const size_t maxwid = 72;
 	const size_t firstline = maxwid - sizeof ("Autocrypt:");
@@ -462,20 +471,21 @@ g_mime_autocrypt_header_to_string (GMimeAutocryptHeader *ah, gboolean gossip)
 		offset = firstlinekeylen;
 	}
 	g_ptr_array_add (lines, first);
-
+	
 	while (offset < ksz) {
 		gsize newsz = MIN((maxwid/4)*3, ksz - offset);
 		g_ptr_array_add (lines, g_base64_encode (kp + offset, newsz));
 		offset += newsz;
 	}
-
+	
 	g_ptr_array_add (lines, NULL);
-
+	
 	char *ret = g_strjoinv (" ", (gchar**)(lines->pdata));
 	g_ptr_array_unref (lines);
 	return ret;
 }
 
+
 /**
  * g_mime_autocrypt_header_compare:
  * @ah1: a #GMimeAutocryptHeader object
@@ -489,7 +499,7 @@ g_mime_autocrypt_header_to_string (GMimeAutocryptHeader *ah, gboolean gossip)
  *  - keydata
  *  - prefer_encrypt
  *
- * Returns: -1, 0, or 1 @ah1 is less than, equal to, or greater than @ah2.
+ * Returns: -1, 0, or 1 when @ah1 is less than, equal to, or greater than @ah2.
  **/
 int
 g_mime_autocrypt_header_compare (GMimeAutocryptHeader *ah1, GMimeAutocryptHeader *ah2)
@@ -515,7 +525,7 @@ g_mime_autocrypt_header_compare (GMimeAutocryptHeader *ah1, GMimeAutocryptHeader
 		if (ret)
 			return ret;
 	}
-
+	
 	if (!ah1->keydata && ah2->keydata)
 		return -1;
 	if (ah1->keydata && !ah2->keydata)
@@ -533,7 +543,7 @@ g_mime_autocrypt_header_compare (GMimeAutocryptHeader *ah1, GMimeAutocryptHeader
 		if (ret)
 			return ret;
 	}
-
+	
 	if (ah1->prefer_encrypt < ah2->prefer_encrypt)
 		return -1;
 	if (ah1->prefer_encrypt > ah2->prefer_encrypt)
@@ -555,26 +565,30 @@ g_mime_autocrypt_header_clone (GMimeAutocryptHeader *dst, GMimeAutocryptHeader *
 {
 	if (!dst->address || !src->address)
 		return;
+	
 	if (g_strcmp0 (internet_address_mailbox_get_idn_addr (dst->address), internet_address_mailbox_get_idn_addr (src->address)))
 		return;
-
+	
 	if (dst->keydata)
 		g_bytes_unref (dst->keydata);
+	
 	if (src->keydata) {
 		g_bytes_ref (src->keydata);
 		dst->keydata = src->keydata;
-	} else
+	} else {
 		dst->keydata = NULL;
-
+	}
+	
 	dst->prefer_encrypt = src->prefer_encrypt;
-
+	
 	if (dst->effective_date)
 		g_date_time_unref (dst->effective_date);
 	if (src->effective_date) {
 		g_date_time_ref (src->effective_date);
 		dst->effective_date = src->effective_date;
-	} else
+	} else {
 		dst->effective_date = NULL;
+	}
 }
 
 
@@ -582,7 +596,7 @@ g_mime_autocrypt_header_clone (GMimeAutocryptHeader *dst, GMimeAutocryptHeader *
 
 
 static void g_mime_autocrypt_header_list_class_init (GMimeAutocryptHeaderListClass *klass);
-static void g_mime_autocrypt_header_list_init (GMimeAutocryptHeaderList *ahl, GMimeAutocryptHeaderListClass *klass);
+static void g_mime_autocrypt_header_list_init (GMimeAutocryptHeaderList *list, GMimeAutocryptHeaderListClass *klass);
 static void g_mime_autocrypt_header_list_finalize (GObject *object);
 
 static GObjectClass *ahl_parent_class = NULL;
@@ -623,17 +637,18 @@ g_mime_autocrypt_header_list_class_init (GMimeAutocryptHeaderListClass *klass)
 }
 
 static void
-g_mime_autocrypt_header_list_init (GMimeAutocryptHeaderList *ahl, GMimeAutocryptHeaderListClass *klass)
+g_mime_autocrypt_header_list_init (GMimeAutocryptHeaderList *list, GMimeAutocryptHeaderListClass *klass)
 {
-	ahl->array = g_ptr_array_new_with_free_func (g_object_unref);
+	list->array = g_ptr_array_new_with_free_func (g_object_unref);
 }
 
 static void
 g_mime_autocrypt_header_list_finalize (GObject *object)
 {
-	GMimeAutocryptHeaderList *ahl = (GMimeAutocryptHeaderList *) object;
-
-	g_ptr_array_unref (ahl->array);
+	GMimeAutocryptHeaderList *list = (GMimeAutocryptHeaderList *) object;
+	
+	g_ptr_array_unref (list->array);
+	
 	G_OBJECT_CLASS (ahl_parent_class)->finalize (object);
 }
 
@@ -654,7 +669,7 @@ g_mime_autocrypt_header_list_new (void)
 
 /**
  * g_mime_autocrypt_header_list_add_missing_addresses:
- * @acheaders: a #GMimeAutocryptHeaderList object
+ * @list: a #GMimeAutocryptHeaderList object
  * @addresses: an #InternetAddressList object
  *
  * Adds a new incomplete #GMimeAutocryptHeader object for each
@@ -663,130 +678,146 @@ g_mime_autocrypt_header_list_new (void)
  * Returns: the number of addresses added
  **/
 guint
-g_mime_autocrypt_header_list_add_missing_addresses (GMimeAutocryptHeaderList *acheaders, InternetAddressList *list)
+g_mime_autocrypt_header_list_add_missing_addresses (GMimeAutocryptHeaderList *list, InternetAddressList *addresses)
 {
-	g_return_val_if_fail (GMIME_IS_AUTOCRYPT_HEADER_LIST (acheaders), 0);
-	guint ret = 0;
+	guint count = 0;
 	guint i;
-	for (i = 0; i < internet_address_list_length (list); i++) {
-		InternetAddress *a = internet_address_list_get_address (list, i);
-		if (INTERNET_ADDRESS_IS_GROUP(a)) {
-			ret += g_mime_autocrypt_header_list_add_missing_addresses (acheaders, internet_address_group_get_members (INTERNET_ADDRESS_GROUP (a)));
-		} else if (INTERNET_ADDRESS_IS_MAILBOX(a)) {
-			InternetAddressMailbox *m = INTERNET_ADDRESS_MAILBOX (a);
-			GMimeAutocryptHeader *ah = g_mime_autocrypt_header_list_get_header_for_address (acheaders, m);
-			if (ah == NULL) {
-				ah = g_mime_autocrypt_header_new ();
-				g_mime_autocrypt_header_set_address (ah, m);
-				g_mime_autocrypt_header_list_add (acheaders, ah);
-				ret += 1;
-			}
+	
+	g_return_val_if_fail (GMIME_IS_AUTOCRYPT_HEADER_LIST (list), 0);
+	g_return_val_if_fail (IS_INTERNET_ADDRESS_LIST (addresses), 0);
+	
+	for (i = 0; i < internet_address_list_length (addresses); i++) {
+		InternetAddress *address = internet_address_list_get_address (addresses, i);
+		
+		if (INTERNET_ADDRESS_IS_GROUP (address)) {
+			InternetAddressGroup *group = (InternetAddressGroup *) address;
+			
+			count += g_mime_autocrypt_header_list_add_missing_addresses (list, internet_address_group_get_members (group));
 		} else {
-			/* FIXME: what do we do here?  what is this
-			   thing if it's neither mailbox nor group?
-			   It should be safe to just ignore it. */
+			InternetAddressMailbox *mailbox = (InternetAddressMailbox *) address;
+			GMimeAutocryptHeader *ah;
+			
+			if (!(ah = g_mime_autocrypt_header_list_get_header_for_address (list, mailbox))) {
+				ah = g_mime_autocrypt_header_new ();
+				g_mime_autocrypt_header_set_address (ah, mailbox);
+				g_mime_autocrypt_header_list_add (list, ah);
+				count++;
+			}
 		}
 	}
-	return ret;
+	
+	return count;
 }
 
 
 /**
  * g_mime_autocrypt_header_list_add:
- * @acheaders: a #GMimeAutocryptHeaderList object
- * @ah: a #GMimeAutocryptHeader object
+ * @list: a #GMimeAutocryptHeaderList object
+ * @header: a #GMimeAutocryptHeader object
  *
  * Adds a the passed #GMimeAutocryptHeader to the list.
  **/
 void
-g_mime_autocrypt_header_list_add (GMimeAutocryptHeaderList *acheaders, GMimeAutocryptHeader *ah)
+g_mime_autocrypt_header_list_add (GMimeAutocryptHeaderList *list, GMimeAutocryptHeader *header)
 {
-	g_return_if_fail (GMIME_IS_AUTOCRYPT_HEADER_LIST (acheaders));
-	g_return_if_fail (GMIME_IS_AUTOCRYPT_HEADER (ah));
-
-	g_object_ref (ah);
-	g_ptr_array_add (acheaders->array, ah);
+	g_return_if_fail (GMIME_IS_AUTOCRYPT_HEADER_LIST (list));
+	g_return_if_fail (GMIME_IS_AUTOCRYPT_HEADER (header));
+	
+	g_object_ref (header);
+	
+	g_ptr_array_add (list->array, header);
 }
 
+
 /**
  * g_mime_autocrypt_header_list_get_count:
- * @acheaders: a #GMimeAutocryptHeaderList object
+ * @list: a #GMimeAutocryptHeaderList object
  *
  * See how many Autocrypt headers are in the list.
  * 
  * Returns: the number of available Autocrypt headers
  **/
 guint
-g_mime_autocrypt_header_list_get_count (GMimeAutocryptHeaderList *acheaders)
+g_mime_autocrypt_header_list_get_count (GMimeAutocryptHeaderList *list)
 {
-	g_return_val_if_fail (GMIME_IS_AUTOCRYPT_HEADER_LIST (acheaders), 0);
-
-	return acheaders->array->len;
+	g_return_val_if_fail (GMIME_IS_AUTOCRYPT_HEADER_LIST (list), 0);
+	
+	return list->array->len;
 }
 
+
 /**
  * g_mime_autocrypt_header_list_get_header_at:
- * @acheaders: a #GMimeAutocryptHeaderList object
- * @n: an index into the list
+ * @list: a #GMimeAutocryptHeaderList object
+ * @index: an index into the list
  *
- * Get the Nth header in the list (returns %NULL on error, or if n is out of bounds)
+ * Get the Nth header in the list (returns %NULL on error, or if @index is out of bounds)
  * 
  * Returns: (transfer none): a pointer to the Nth header in the list.
  **/
 GMimeAutocryptHeader *
-g_mime_autocrypt_header_list_get_header_at (GMimeAutocryptHeaderList *acheaders, guint n)
+g_mime_autocrypt_header_list_get_header_at (GMimeAutocryptHeaderList *list, guint index)
 {
-	g_return_val_if_fail (GMIME_IS_AUTOCRYPT_HEADER_LIST (acheaders), NULL);
-
-	if (n < acheaders->array->len)
-		return (GMimeAutocryptHeader*)(acheaders->array->pdata[n]);
-
-	return NULL;
+	g_return_val_if_fail (GMIME_IS_AUTOCRYPT_HEADER_LIST (list), NULL);
+	
+	if (index >= list->array->len)
+		return NULL;
+	
+	return (GMimeAutocryptHeader *) list->array->pdata[index];
 }
 
+
 /**
  * g_mime_autocrypt_header_list_get_header_for_address:
- * @acheaders: a #GMimeAutocryptHeaderList object
- * @addr: an #InternetAddressMailbox object
+ * @list: a #GMimeAutocryptHeaderList object
+ * @mailbox: an #InternetAddressMailbox object
  *
- * returns an Autocrypt header corresponding to the given
- * InternetAddressMailbox.
+ * Gets the Autocrypt header corresponding to the given @mailbox.
  * 
  * Returns: (transfer none): a pointer to the header in the list which
  * matches the requested address, or %NULL if no such header exists in
  * the list.
  **/
 GMimeAutocryptHeader *
-g_mime_autocrypt_header_list_get_header_for_address (GMimeAutocryptHeaderList *acheaders, InternetAddressMailbox *addr)
+g_mime_autocrypt_header_list_get_header_for_address (GMimeAutocryptHeaderList *list, InternetAddressMailbox *mailbox)
 {
-	g_return_val_if_fail (GMIME_IS_AUTOCRYPT_HEADER_LIST (acheaders), NULL);
-	g_return_val_if_fail (INTERNET_ADDRESS_IS_MAILBOX (addr), NULL);
-
+	const char *idn;
 	guint i;
-	for (i = 0; i < acheaders->array->len; i++) {
-		GMimeAutocryptHeader* ah = (GMimeAutocryptHeader*)(acheaders->array->pdata[i]);
-		if (g_strcmp0 (internet_address_mailbox_get_idn_addr (addr), internet_address_mailbox_get_idn_addr (ah->address)) == 0)
+	
+	g_return_val_if_fail (GMIME_IS_AUTOCRYPT_HEADER_LIST (list), NULL);
+	g_return_val_if_fail (INTERNET_ADDRESS_IS_MAILBOX (mailbox), NULL);
+	
+	idn = internet_address_mailbox_get_idn_addr (mailbox);
+	
+	for (i = 0; i < list->array->len; i++) {
+		GMimeAutocryptHeader *ah = (GMimeAutocryptHeader *) list->array->pdata[i];
+		const char *addr = internet_address_mailbox_get_idn_addr (ah->address);
+		
+		if (!g_strcmp0 (idn, addr))
 			return ah;
 	}
+	
 	return NULL;
 }
 
 /**
  * g_mime_autocrypt_header_list_remove_incomplete:
- * @acheaders: a #GMimeAutocryptHeaderList object
+ * @list: a #GMimeAutocryptHeaderList object
  *
  * Remove all incomplete Autocrypt headers from the list.
  **/
 void
-g_mime_autocrypt_header_list_remove_incomplete (GMimeAutocryptHeaderList *acheaders)
+g_mime_autocrypt_header_list_remove_incomplete (GMimeAutocryptHeaderList *list)
 {
-	g_return_if_fail (GMIME_IS_AUTOCRYPT_HEADER_LIST (acheaders));
-
 	guint i;
-	for (i = 0; i < acheaders->array->len; i++) {
-		GMimeAutocryptHeader* ah = (GMimeAutocryptHeader*)(acheaders->array->pdata[i]);
+	
+	g_return_if_fail (GMIME_IS_AUTOCRYPT_HEADER_LIST (list));
+	
+	for (i = 0; i < list->array->len; i++) {
+		GMimeAutocryptHeader *ah = (GMimeAutocryptHeader *) list->array->pdata[i];
+		
 		if (!g_mime_autocrypt_header_is_complete (ah)) {
-			g_ptr_array_remove_index (acheaders->array, i);
+			g_ptr_array_remove_index (list->array, i);
 			i--;
 		}
 	}
