@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2013 Jeffrey Stedfast
+ *  Copyright (C) 2000-2017 Jeffrey Stedfast
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public License
@@ -117,7 +117,7 @@ g_mime_data_wrapper_finalize (GObject *object)
 GMimeDataWrapper *
 g_mime_data_wrapper_new (void)
 {
-	return g_object_newv (GMIME_TYPE_DATA_WRAPPER, 0, NULL);
+	return g_object_new (GMIME_TYPE_DATA_WRAPPER, NULL);
 }
 
 
