@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2014 Jeffrey Stedfast
+ *  Copyright (C) 2000-2017 Jeffrey Stedfast
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public License
@@ -37,41 +37,9 @@ typedef struct _GMimeGpgContext GMimeGpgContext;
 typedef struct _GMimeGpgContextClass GMimeGpgContextClass;
 
 
-/**
- * GMimeGpgContext:
- * @parent_object: parent #GMimeCryptoContext
- * @always_trust: %TRUE if keys should always be trusted
- * @path: path to gpg
- *
- * A GnuPG crypto context.
- **/
-struct _GMimeGpgContext {
-	GMimeCryptoContext parent_object;
-	gboolean auto_key_retrieve;
-	gboolean always_trust;
-	gboolean use_agent;
-	char *path;
-};
-
-struct _GMimeGpgContextClass {
-	GMimeCryptoContextClass parent_class;
-	
-};
-
-
 GType g_mime_gpg_context_get_type (void);
 
-
-GMimeCryptoContext *g_mime_gpg_context_new (GMimePasswordRequestFunc request_passwd, const char *path);
-
-gboolean g_mime_gpg_context_get_auto_key_retrieve (GMimeGpgContext *ctx);
-void g_mime_gpg_context_set_auto_key_retrieve (GMimeGpgContext *ctx, gboolean auto_key_retrieve);
-
-gboolean g_mime_gpg_context_get_always_trust (GMimeGpgContext *ctx);
-void g_mime_gpg_context_set_always_trust (GMimeGpgContext *ctx, gboolean always_trust);
-
-gboolean g_mime_gpg_context_get_use_agent (GMimeGpgContext *ctx);
-void g_mime_gpg_context_set_use_agent (GMimeGpgContext *ctx, gboolean use_agent);
+GMimeCryptoContext *g_mime_gpg_context_new (void);
 
 G_END_DECLS
 
