@@ -81,16 +81,19 @@ char *g_mime_header_format_default (GMimeHeader *header, GMimeFormatOptions *opt
  * A message or mime-part header.
  **/
 struct _GMimeHeader {
+	/* <private> */
 	GObject parent_object;
 	
 	char *name, *value;
 	
-	/* < private > */
+	/* <private> */
 	GMimeHeaderRawValueFormatter formatter;
 	GMimeParserOptions *options;
+	gboolean reformat;
 	gpointer changed;
 	char *raw_value;
 	char *raw_name;
+	char *charset;
 	gint64 offset;
 };
 
