@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2014 Jeffrey Stedfast
+ *  Copyright (C) 2000-2017 Jeffrey Stedfast
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public License
@@ -29,6 +29,7 @@
 #include "gmime-message-partial.h"
 #include "gmime-stream-cat.h"
 #include "gmime-stream-mem.h"
+#include "gmime-internal.h"
 #include "gmime-parser.h"
 
 
@@ -36,23 +37,17 @@
  * SECTION: gmime-message-partial
  * @title: GMimeMessagePartial
  * @short_description: Partial MIME parts
- * @se_also:
+ * @see_also:
  *
  * A #GMimeMessagePartial represents the message/partial MIME part.
  **/
 
-
 /* GObject class methods */
 static void g_mime_message_partial_class_init (GMimeMessagePartialClass *klass);
 static void g_mime_message_partial_init (GMimeMessagePartial *catpart, GMimeMessagePartialClass *klass);
 static void g_mime_message_partial_finalize (GObject *object);
 
 /* GMimeObject class methods */
-static void message_partial_prepend_header (GMimeObject *object, const char *header, const char *value);
-static void message_partial_append_header (GMimeObject *object, const char *header, const char *value);
-static void message_partial_set_header (GMimeObject *object, const char *header, const char *value);
-static const char *message_partial_get_header (GMimeObject *object, const char *header);
-static gboolean message_partial_remove_header (GMimeObject *object, const char *header);
 static void message_partial_set_content_type (GMimeObject *object, GMimeContentType *content_type);
 
 
@@ -94,11 +89,6 @@ g_mime_message_partial_class_init (GMimeMessagePartialClass *klass)
 	
 	gobject_class->finalize = g_mime_message_partial_finalize;
 	
-	object_class->prepend_header = message_partial_prepend_header;
-	object_class->append_header = message_partial_append_header;
-	object_class->remove_header = message_partial_remove_header;
-	object_class->set_header = message_partial_set_header;
-	object_class->get_header = message_partial_get_header;
 	object_class->set_content_type = message_partial_set_content_type;
 }
 
@@ -120,47 +110,6 @@ g_mime_message_partial_finalize (GObject *object)
 	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
-static void
-message_partial_prepend_header (GMimeObject *object, const char *header, const char *value)
-{
-	/* RFC 1864 states that you cannot set a Content-MD5 on a message part */
-	if (!g_ascii_strcasecmp ("Content-MD5", header))
-		return;
-	
-	GMIME_OBJECT_CLASS (parent_class)->prepend_header (object, header, value);
-}
-
-static void
-message_partial_append_header (GMimeObject *object, const char *header, const char *value)
-{
-	/* RFC 1864 states that you cannot set a Content-MD5 on a message part */
-	if (!g_ascii_strcasecmp ("Content-MD5", header))
-		return;
-	
-	GMIME_OBJECT_CLASS (parent_class)->append_header (object, header, value);
-}
-
-static void
-message_partial_set_header (GMimeObject *object, const char *header, const char *value)
-{
-	/* RFC 1864 states that you cannot set a Content-MD5 on a message part */
-	if (!g_ascii_strcasecmp ("Content-MD5", header))
-		return;
-	
-	GMIME_OBJECT_CLASS (parent_class)->set_header (object, header, value);
-}
-
-static const char *
-message_partial_get_header (GMimeObject *object, const char *header)
-{
-	return GMIME_OBJECT_CLASS (parent_class)->get_header (object, header);
-}
-
-static gboolean
-message_partial_remove_header (GMimeObject *object, const char *header)
-{
-	return GMIME_OBJECT_CLASS (parent_class)->remove_header (object, header);
-}
 
 static void
 message_partial_set_content_type (GMimeObject *object, GMimeContentType *content_type)
@@ -199,7 +148,7 @@ g_mime_message_partial_new (const char *id, int number, int total)
 	GMimeMessagePartial *partial;
 	char *num;
 	
-	partial = g_object_newv (GMIME_TYPE_MESSAGE_PARTIAL, 0, NULL);
+	partial = g_object_new (GMIME_TYPE_MESSAGE_PARTIAL, NULL);
 	
 	content_type = g_mime_content_type_new ("message", "partial");
 	
@@ -216,7 +165,7 @@ g_mime_message_partial_new (const char *id, int number, int total)
 	g_mime_content_type_set_parameter (content_type, "total", num);
 	g_free (num);
 	
-	g_mime_object_set_content_type (GMIME_OBJECT (partial), content_type);
+	g_mime_object_set_content_type ((GMimeObject *) partial, content_type);
 	g_object_unref (content_type);
 	
 	return partial;
@@ -345,18 +294,18 @@ g_mime_message_partial_reconstruct_message (GMimeMessagePartial **partials, size
 		if ((size_t) number != i + 1)
 			goto exception;
 		
-		wrapper = g_mime_part_get_content_object (GMIME_PART (partial));
+		wrapper = g_mime_part_get_content ((GMimePart *) partial);
 		stream = g_mime_data_wrapper_get_stream (wrapper);
 		
 		g_mime_stream_reset (stream);
-		g_mime_stream_cat_add_source (GMIME_STREAM_CAT (cat), stream);
+		g_mime_stream_cat_add_source ((GMimeStreamCat *) cat, stream);
 	}
 	
 	parser = g_mime_parser_new ();
 	g_mime_parser_init_with_stream (parser, cat);
 	g_object_unref (cat);
 	
-	message = g_mime_parser_construct_message (parser);
+	message = g_mime_parser_construct_message (parser, NULL);
 	g_object_unref (parser);
 	
 	return message;
@@ -372,21 +321,27 @@ g_mime_message_partial_reconstruct_message (GMimeMessagePartial **partials, size
 static GMimeMessage *
 message_partial_message_new (GMimeMessage *base)
 {
-	const char *name, *value;
+	const char *name, *raw_name, *raw_value;
+	GMimeHeaderList *headers;
 	GMimeMessage *message;
-	GMimeHeaderList *list;
-	GMimeHeaderIter iter;
+	GMimeHeader *header;
+	gint64 offset;
+	int count, i;
 	
 	message = g_mime_message_new (FALSE);
 	
-	list = ((GMimeObject *) base)->headers;
+	headers = ((GMimeObject *) base)->headers;
 	
-	if (g_mime_header_list_get_iter (list, &iter)) {
-		do {
-			name = g_mime_header_iter_get_name (&iter);
-			value = g_mime_header_iter_get_value (&iter);
-			g_mime_object_append_header ((GMimeObject *) message, name, value);
-		} while (g_mime_header_iter_next (&iter));
+	count = g_mime_header_list_get_count (headers);
+	
+	for (i = 0; i < count; i++) {
+		header = g_mime_header_list_get_header_at (headers, i);
+		raw_value = g_mime_header_get_raw_value (header);
+		raw_name = g_mime_header_get_raw_name (header);
+		offset = g_mime_header_get_offset (header);
+		name = g_mime_header_get_name (header);
+		
+		_g_mime_object_append_header ((GMimeObject *) message, name, raw_name, raw_value, offset);
 	}
 	
 	return message;
@@ -397,14 +352,14 @@ message_partial_message_new (GMimeMessage *base)
  * g_mime_message_partial_split_message:
  * @message: message object
  * @max_size: max size
- * @nparts: number of parts
+ * @nparts: (out): number of parts
  *
  * Splits @message into an array of #GMimeMessage objects each
  * containing a single #GMimeMessagePartial object containing
  * @max_size bytes or fewer. @nparts is set to the number of
  * #GMimeMessagePartial objects created.
  *
- * Returns: (transfer full): an array of #GMimeMessage objects and
+ * Returns: (nullable) (transfer full): an array of #GMimeMessage objects and
  * sets @nparts to the number of messages returned or %NULL on fail.
  **/
 GMimeMessage **
@@ -413,6 +368,7 @@ g_mime_message_partial_split_message (GMimeMessage *message, size_t max_size, si
 	GMimeMessage **messages;
 	GMimeMessagePartial *partial;
 	GMimeStream *stream, *substream;
+	GMimeFormatOptions *options;
 	GMimeDataWrapper *wrapper;
 	const unsigned char *buf;
 	GPtrArray *parts;
@@ -425,8 +381,10 @@ g_mime_message_partial_split_message (GMimeMessage *message, size_t max_size, si
 	
 	g_return_val_if_fail (GMIME_IS_MESSAGE (message), NULL);
 	
+	options = g_mime_format_options_get_default ();
 	stream = g_mime_stream_mem_new ();
-	if (g_mime_object_write_to_stream (GMIME_OBJECT (message), stream) == -1) {
+	
+	if (g_mime_object_write_to_stream ((GMimeObject *) message, options, stream) == -1) {
 		g_object_unref (stream);
 		return NULL;
 	}
@@ -436,7 +394,7 @@ g_mime_message_partial_split_message (GMimeMessage *message, size_t max_size, si
 	len = g_mime_stream_length (stream);
 	
 	/* optimization */
-	if (len <= max_size) {
+	if (len <= (gint64) max_size) {
 		g_object_unref (stream);
 		g_object_ref (message);
 		
@@ -474,14 +432,14 @@ g_mime_message_partial_split_message (GMimeMessage *message, size_t max_size, si
 	
 	for (i = 0; i < parts->len; i++) {
 		partial = g_mime_message_partial_new (id, i + 1, parts->len);
-		wrapper = g_mime_data_wrapper_new_with_stream (GMIME_STREAM (parts->pdata[i]),
+		wrapper = g_mime_data_wrapper_new_with_stream ((GMimeStream *) parts->pdata[i],
 							       GMIME_CONTENT_ENCODING_DEFAULT);
 		g_object_unref (parts->pdata[i]);
-		g_mime_part_set_content_object (GMIME_PART (partial), wrapper);
+		g_mime_part_set_content ((GMimePart *) partial, wrapper);
 		g_object_unref (wrapper);
 		
 		parts->pdata[i] = message_partial_message_new (message);
-		g_mime_message_set_mime_part (GMIME_MESSAGE (parts->pdata[i]), GMIME_OBJECT (partial));
+		g_mime_message_set_mime_part ((GMimeMessage *) parts->pdata[i], (GMimeObject *) partial);
 		g_object_unref (partial);
 	}
 	
