@@ -31,6 +31,7 @@
 #include <gmime/gmime-header.h>
 #include <gmime/gmime-stream.h>
 #include <gmime/gmime-autocrypt.h>
+#include <gmime/gmime-crypto-context.h>
 
 G_BEGIN_DECLS
 
@@ -86,7 +87,7 @@ struct _GMimeMessage {
 	GDateTime *date;
 	char *subject;
 	
-	/* < private > */
+	/* <private> */
 	char *marker;
 };
 
@@ -123,8 +124,9 @@ const char *g_mime_message_get_message_id (GMimeMessage *message);
 GMimeObject *g_mime_message_get_mime_part (GMimeMessage *message);
 void g_mime_message_set_mime_part (GMimeMessage *message, GMimeObject *mime_part);
 
-GMimeAutocryptHeaderList *g_mime_message_get_autocrypt_headers (GMimeMessage *part, gint actype, GDateTime *now);
-GMimeAutocryptHeaderList *g_mime_message_get_autocrypt_gossip_headers (GMimeMessage *part, gint actype, GDateTime *now);
+GMimeAutocryptHeader *g_mime_message_get_autocrypt_header (GMimeMessage *message, GDateTime *now);
+GMimeAutocryptHeaderList *g_mime_message_get_autocrypt_gossip_headers (GMimeMessage *message, GDateTime *now, GMimeDecryptFlags flags, const char *session_key, GError **err);
+GMimeAutocryptHeaderList *g_mime_message_get_autocrypt_gossip_headers_from_inner_part (GMimeMessage *message, GDateTime *now, GMimeObject *inner_part);
 
 /* convenience functions */
 
