@@ -59,29 +59,7 @@
  * into multiple #GMimeMessage objects.
  **/
 
-typedef enum {
-	OPENPGP_NONE                     = 0,
-	OPENPGP_BEGIN_PGP_MESSAGE        = (1 << 0),
-	OPENPGP_END_PGP_MESSAGE          = (1 << 1) | (1 << 0),
-	OPENPGP_BEGIN_PGP_SIGNED_MESSAGE = (1 << 2),
-	OPENPGP_BEGIN_PGP_SIGNATURE      = (1 << 3) | (1 << 2),
-	OPENPGP_END_PGP_SIGNATURE        = (1 << 4) | (1 << 3) | (1 << 2)
-} openpgp_state_t;
-
-typedef struct {
-	const char *marker;
-	size_t len;
-	openpgp_state_t before;
-	openpgp_state_t after;
-} GMimeOpenPGPMarker;
-
-static const GMimeOpenPGPMarker openpgp_markers[] = {
-	{ "-----BEGIN PGP MESSAGE-----",        27, OPENPGP_NONE,                     OPENPGP_BEGIN_PGP_MESSAGE },
-	{ "-----END PGP MESSAGE-----",          25, OPENPGP_BEGIN_PGP_MESSAGE,        OPENPGP_END_PGP_MESSAGE   },
-	{ "-----BEGIN PGP SIGNED MESSAGE-----", 34, OPENPGP_NONE,                     OPENPGP_BEGIN_PGP_SIGNED_MESSAGE },
-	{ "-----BEGIN PGP SIGNATURE-----",      29, OPENPGP_BEGIN_PGP_SIGNED_MESSAGE, OPENPGP_BEGIN_PGP_SIGNATURE },
-	{ "-----END PGP SIGNATURE-----",        27, OPENPGP_BEGIN_PGP_SIGNATURE,      OPENPGP_END_PGP_SIGNATURE }
-};
+extern const GMimeOpenPGPMarker g_mime_openpgp_markers[9];
 
 typedef enum {
 	BOUNDARY_NONE,
@@ -188,7 +166,7 @@ struct _GMimeParserPrivate {
 	
 	BoundaryStack *bounds;
 	
-	openpgp_state_t openpgp;
+	GMimeOpenPGPState openpgp;
 	short int state;
 	
 	unsigned short int midline:1;
@@ -263,8 +241,8 @@ parser_find_header (GMimeParser *parser, const char *name, gint64 *offset)
 	Header *header;
 	guint i;
 	
-	for (i = 0; i < priv->headers->len; i++) {
-		header = priv->headers->pdata[i];
+	for (i = priv->headers->len; i > 0; i--) {
+		header = priv->headers->pdata[i - 1];
 		
 		if (g_ascii_strcasecmp (header->name, name) != 0)
 			continue;
@@ -403,7 +381,7 @@ parser_init (GMimeParser *parser, GMimeStream *stream)
 	
 	priv->header_offset = -1;
 	
-	priv->openpgp = OPENPGP_NONE;
+	priv->openpgp = GMIME_OPENPGP_NONE;
 	
 	priv->midline = FALSE;
 	priv->seekable = offset != -1;
@@ -869,11 +847,25 @@ next_alloc_size (size_t n)
 	priv->headerleft -= len;                                          \
 } G_STMT_END
 
-static void
-header_parse (GMimeParser *parser)
+static inline gboolean
+is_7bit_clean (const gchar *str)
 {
+	while (*str != '\0') {
+		if ((*str & 0x80) != 0)
+			return FALSE;
+		
+		str++;
+	}
+	
+	return TRUE;
+}
+
+static void
+header_parse (GMimeParser *parser, GMimeParserOptions *options)
+{
+	gboolean can_warn = g_mime_parser_options_get_warning_callback (options) != NULL;
 	struct _GMimeParserPrivate *priv = parser->priv;
-	gboolean valid = TRUE, blank = FALSE;
+	gboolean blank = FALSE;
 	register char *inptr;
 	Header *header;
 	
@@ -887,7 +879,6 @@ header_parse (GMimeParser *parser)
 		if (is_blank (*inptr)) {
 			blank = TRUE;
 		} else if (blank || is_ctrl (*inptr)) {
-			valid = FALSE;
 			break;
 		}
 		
@@ -898,6 +889,7 @@ header_parse (GMimeParser *parser)
 	
 	if (*inptr != ':') {
 		/* ignore invalid headers */
+		_g_mime_parser_options_warn (options, priv->header_offset, GMIME_WARN_INVALID_HEADER, priv->headerbuf);
 		w(g_warning ("Invalid header at %lld: '%s'",
 			     (long long) priv->header_offset,
 			     priv->headerbuf));
@@ -929,6 +921,9 @@ header_parse (GMimeParser *parser)
 	if (priv->regex && g_regex_match (priv->regex, header->name, 0, NULL))
 		priv->header_cb (parser, header->name, header->raw_value,
 				 header->offset, priv->user_data);
+	
+	if (can_warn && (!is_7bit_clean (header->name) || !g_utf8_validate (header->raw_value, -1, NULL)))
+		_g_mime_parser_options_warn (options, header->offset, GMIME_WARN_UNENCODED_8BIT_HEADER, header->name);
 }
 
 enum {
@@ -981,7 +976,7 @@ has_content_headers (GPtrArray *headers)
 }
 
 static int
-parser_step_headers (GMimeParser *parser)
+parser_step_headers (GMimeParser *parser, GMimeParserOptions *options)
 {
 	struct _GMimeParserPrivate *priv = parser->priv;
 	gboolean eoln, valid = TRUE, fieldname = TRUE;
@@ -997,6 +992,9 @@ parser_step_headers (GMimeParser *parser)
 	priv->headers_begin = parser_offset (priv, NULL);
 	priv->header_offset = priv->headers_begin;
 	
+	priv->headerleft += priv->headerptr - priv->headerbuf;
+	priv->headerptr = priv->headerbuf;
+	
 	inptr = priv->inptr;
 	inend = priv->inend;
 	
@@ -1017,7 +1015,7 @@ parser_step_headers (GMimeParser *parser)
 			
 			/* if we are scanning a new line, check for a folded header */
 			if (!priv->midline && continuation && (*inptr != ' ' && *inptr != '\t')) {
-				header_parse (parser);
+				header_parse (parser, options);
 				priv->header_offset = parser_offset (priv, inptr);
 				continuation = FALSE;
 				fieldname = TRUE;
@@ -1114,8 +1112,6 @@ parser_step_headers (GMimeParser *parser)
 				goto refill;
 			}
 			
-			header_append (priv, start, len);
-			
 			if (inptr > start && inptr[-1] == '\r')
 				len--;
 			
@@ -1123,8 +1119,13 @@ parser_step_headers (GMimeParser *parser)
 			if (!priv->midline && len == 0)
 				goto headers_end;
 			
-			/* inptr has to be less than inend - 1 */
-			header_append (priv, "\n", 1);
+			if (inptr > start && inptr[-1] == '\r')
+				len++;
+			
+			/* increment len to include the \n */
+			len++;
+			
+			header_append (priv, start, len);
 			priv->midline = FALSE;
 			continuation = TRUE;
 			inptr++;
@@ -1144,7 +1145,7 @@ parser_step_headers (GMimeParser *parser)
  headers_end:
 	
 	if (priv->headerptr > priv->headerbuf)
-		header_parse (parser);
+		header_parse (parser, options);
 	
 	priv->headers_end = parser_offset (priv, start);
 	priv->state = GMIME_PARSER_STATE_HEADERS_END;
@@ -1254,7 +1255,7 @@ parser_skip_line (GMimeParser *parser)
 }
 
 static int
-parser_step (GMimeParser *parser)
+parser_step (GMimeParser *parser, GMimeParserOptions *options)
 {
 	struct _GMimeParserPrivate *priv = parser->priv;
 	
@@ -1283,7 +1284,7 @@ parser_step (GMimeParser *parser)
 		break;
 	case GMIME_PARSER_STATE_MESSAGE_HEADERS:
 	case GMIME_PARSER_STATE_HEADERS:
-		parser_step_headers (parser);
+		parser_step_headers (parser, options);
 		
 		if (priv->message_headers_begin == -1) {
 			priv->message_headers_begin = priv->headers_begin;
@@ -1391,13 +1392,13 @@ check_boundary (struct _GMimeParserPrivate *priv, const char *start, size_t len)
 		len -= 2;
 		
 		/* check for OpenPGP markers... */
-		for (i = 0; i < G_N_ELEMENTS (openpgp_markers); i++) {
-			const char *marker = openpgp_markers[i].marker + 2;
-			openpgp_state_t state = openpgp_markers[i].before;
-			size_t n = openpgp_markers[i].len - 2;
+		for (i = 0; i < G_N_ELEMENTS (g_mime_openpgp_markers); i++) {
+			const char *pgp_marker = g_mime_openpgp_markers[i].marker + 2;
+			GMimeOpenPGPState state = g_mime_openpgp_markers[i].before;
+			size_t n = g_mime_openpgp_markers[i].len - 2;
 			
-			if (len == n && priv->openpgp == state && !strncmp (marker, start, len))
-				priv->openpgp = openpgp_markers[i].after;
+			if (len == n && priv->openpgp == state && !strncmp (pgp_marker, start, len))
+				priv->openpgp = g_mime_openpgp_markers[i].after;
 		}
 	}
 	
@@ -1450,7 +1451,7 @@ parser_scan_content (GMimeParser *parser, GMimeStream *content, gboolean *empty)
 	
 	d(printf ("scan-content\n"));
 	
-	priv->openpgp = OPENPGP_NONE;
+	priv->openpgp = GMIME_OPENPGP_NONE;
 	priv->midline = FALSE;
 	
 	g_assert (priv->inptr <= priv->inend);
@@ -1481,7 +1482,7 @@ parser_scan_content (GMimeParser *parser, GMimeStream *content, gboolean *empty)
 		priv->midline = FALSE;
 		
 		while (inptr < inend) {
-			aligned = (char *) (((long) (inptr + 3)) & ~3);
+			aligned = (char *) (((size_t) (inptr + 3)) & ~3);
 			start = inptr;
 			
 			/* Note: see optimization comment [1] */
@@ -1559,6 +1560,7 @@ static void
 parser_scan_mime_part_content (GMimeParser *parser, GMimePart *mime_part, BoundaryType *found)
 {
 	struct _GMimeParserPrivate *priv = parser->priv;
+	GMimeContentType *content_type;
 	GMimeContentEncoding encoding;
 	GMimeDataWrapper *content;
 	GMimeStream *stream;
@@ -1573,6 +1575,7 @@ parser_scan_mime_part_content (GMimeParser *parser, GMimePart *mime_part, Bounda
 		start = parser_offset (priv, NULL);
 	} else {
 		stream = g_mime_stream_mem_new ();
+		start = 0;
 	}
 	
 	*found = parser_scan_content (parser, stream, &empty);
@@ -1595,10 +1598,33 @@ parser_scan_mime_part_content (GMimeParser *parser, GMimePart *mime_part, Bounda
 	g_mime_part_set_content (mime_part, content);
 	g_object_unref (content);
 	
-	if (priv->openpgp == OPENPGP_END_PGP_SIGNATURE)
+	content_type = g_mime_object_get_content_type ((GMimeObject *) mime_part);
+	
+	switch (priv->openpgp) {
+	case GMIME_OPENPGP_END_PGP_SIGNATURE:
 		g_mime_part_set_openpgp_data (mime_part, GMIME_OPENPGP_DATA_SIGNED);
-	else if (priv->openpgp == OPENPGP_END_PGP_MESSAGE)
+		break;
+	case GMIME_OPENPGP_END_PGP_MESSAGE:
 		g_mime_part_set_openpgp_data (mime_part, GMIME_OPENPGP_DATA_ENCRYPTED);
+		break;
+	case GMIME_OPENPGP_END_PGP_PUBLIC_KEY_BLOCK:
+		g_mime_part_set_openpgp_data (mime_part, GMIME_OPENPGP_DATA_PUBLIC_KEY);
+		break;
+	case GMIME_OPENPGP_END_PGP_PRIVATE_KEY_BLOCK:
+		g_mime_part_set_openpgp_data (mime_part, GMIME_OPENPGP_DATA_PRIVATE_KEY);
+		break;
+	default:
+		if (encoding != GMIME_CONTENT_ENCODING_BASE64 && encoding != GMIME_CONTENT_ENCODING_UUENCODE) {
+			/* If the Content-Transfer-Encoding is not obfuscated by base64 or uuencode,
+			   then we can safely assume that no OpenPGP blocks exist. */
+			g_mime_part_set_openpgp_data (mime_part, GMIME_OPENPGP_DATA_NONE);
+		} else if (!g_mime_content_type_is_type (content_type, "text", "*")) {
+			/* The content is base64 or uuencoded, but it's not text so it should be safe
+			   to assume that there won't be any OpenPGP blocks in the decoded content. */
+			g_mime_part_set_openpgp_data (mime_part, GMIME_OPENPGP_DATA_NONE);
+		}
+		break;
+	}
 }
 
 static void
@@ -1608,7 +1634,6 @@ parser_scan_message_part (GMimeParser *parser, GMimeParserOptions *options, GMim
 	ContentType *content_type;
 	GMimeMessage *message;
 	GMimeObject *object;
-	GMimeStream *stream;
 	Header *header;
 	guint i;
 	
@@ -1655,7 +1680,7 @@ parser_scan_message_part (GMimeParser *parser, GMimeParserOptions *options, GMim
 	
 	/* get the headers */
 	priv->state = GMIME_PARSER_STATE_HEADERS;
-	if (parser_step (parser) == GMIME_PARSER_STATE_ERROR) {
+	if (parser_step (parser, options) == GMIME_PARSER_STATE_ERROR) {
 		/* Note: currently cannot happen because
 		 * parser_step_headers() never returns error */
 		*found = BOUNDARY_EOS;
@@ -1689,6 +1714,19 @@ parser_scan_message_part (GMimeParser *parser, GMimeParserOptions *options, GMim
 	g_object_unref (message);
 }
 
+static void
+check_content_header_conflict (GMimeParserOptions *options, GMimeObject *object, const Header *header)
+{
+	const GMimeHeader *existing;
+	
+	if ((existing = g_mime_header_list_get_header (object->headers, header->name)) != NULL) {
+		if (strcmp (existing->raw_value, header->raw_value) != 0)
+			_g_mime_parser_options_warn (options, header->offset, GMIME_CRIT_CONFLICTING_CONTENT_HDR, header->name);
+		else
+			_g_mime_parser_options_warn (options, header->offset, GMIME_WARN_DUPLICATED_CONTENT_HDR, header->name);
+	}
+}
+
 static GMimeObject *
 parser_construct_leaf_part (GMimeParser *parser, GMimeParserOptions *options, ContentType *content_type, gboolean toplevel, BoundaryType *found)
 {
@@ -1713,6 +1751,7 @@ parser_construct_leaf_part (GMimeParser *parser, GMimeParserOptions *options, Co
 		header = priv->headers->pdata[i];
 		
 		if (!toplevel || !g_ascii_strncasecmp (header->name, "Content-", 8)) {
+			check_content_header_conflict (options, object, header);
 			_g_mime_object_append_header (object, header->name, header->raw_name,
 						      header->raw_value, header->offset);
 		}
@@ -1722,7 +1761,7 @@ parser_construct_leaf_part (GMimeParser *parser, GMimeParserOptions *options, Co
 	
 	if (priv->state == GMIME_PARSER_STATE_HEADERS_END) {
 		/* skip empty line after headers */
-		if (parser_step (parser) == GMIME_PARSER_STATE_ERROR) {
+		if (parser_step (parser, options) == GMIME_PARSER_STATE_ERROR) {
 			*found = BOUNDARY_EOS;
 			return object;
 		}
@@ -1732,7 +1771,7 @@ parser_construct_leaf_part (GMimeParser *parser, GMimeParserOptions *options, Co
 		parser_scan_message_part (parser, options, (GMimeMessagePart *) object, found);
 	else
 		parser_scan_mime_part_content (parser, (GMimePart *) object, found);
-	
+
 	return object;
 }
 
@@ -1813,7 +1852,7 @@ parser_scan_multipart_subparts (GMimeParser *parser, GMimeParserOptions *options
 		
 		/* get the headers */
 		priv->state = GMIME_PARSER_STATE_HEADERS;
-		if (parser_step (parser) == GMIME_PARSER_STATE_ERROR) {
+		if (parser_step (parser, options) == GMIME_PARSER_STATE_ERROR) {
 			found = BOUNDARY_EOS;
 			break;
 		}
@@ -1842,9 +1881,9 @@ parser_construct_multipart (GMimeParser *parser, GMimeParserOptions *options, Co
 {
 	struct _GMimeParserPrivate *priv = parser->priv;
 	GMimeMultipart *multipart;
+	gint64 ctype_offset = -1;
 	const char *boundary;
 	GMimeObject *object;
-	GMimeStream *stream;
 	Header *header;
 	guint i;
 	
@@ -1856,6 +1895,11 @@ parser_construct_multipart (GMimeParser *parser, GMimeParserOptions *options, Co
 		header = priv->headers->pdata[i];
 		
 		if (!toplevel || !g_ascii_strncasecmp (header->name, "Content-", 8)) {
+			check_content_header_conflict (options, object, header);
+			
+			if (!g_ascii_strcasecmp (header->name, "Content-Type"))
+				ctype_offset = header->offset;
+			
 			_g_mime_object_append_header (object, header->name, header->raw_name,
 						      header->raw_value, header->offset);
 		}
@@ -1867,7 +1911,7 @@ parser_construct_multipart (GMimeParser *parser, GMimeParserOptions *options, Co
 	
 	if (priv->state == GMIME_PARSER_STATE_HEADERS_END) {
 		/* skip empty line after headers */
-		if (parser_step (parser) == GMIME_PARSER_STATE_ERROR) {
+		if (parser_step (parser, options) == GMIME_PARSER_STATE_ERROR) {
 			*found = BOUNDARY_EOS;
 			return object;
 		}
@@ -1890,6 +1934,12 @@ parser_construct_multipart (GMimeParser *parser, GMimeParserOptions *options, Co
 			return object;
 		}
 		
+		if ((*found == BOUNDARY_PARENT) || (*found == BOUNDARY_PARENT_END))
+			_g_mime_parser_options_warn (options, ctype_offset, GMIME_WARN_MALFORMED_MULTIPART, content_type->subtype);
+		
+		if (*found == BOUNDARY_EOS)
+			_g_mime_parser_options_warn (options, -1, GMIME_WARN_TRUNCATED_MESSAGE, NULL);
+		
 		multipart->write_end_boundary = FALSE;
 		parser_pop_boundary (parser);
 		
@@ -1898,6 +1948,7 @@ parser_construct_multipart (GMimeParser *parser, GMimeParserOptions *options, Co
 		else if (*found == BOUNDARY_PARENT && found_immediate_boundary (priv, FALSE))
 			*found = BOUNDARY_IMMEDIATE;
 	} else {
+		_g_mime_parser_options_warn (options, ctype_offset, GMIME_CRIT_MULTIPART_WITHOUT_BOUNDARY, content_type->subtype);
 		w(g_warning ("multipart without boundary encountered"));
 		/* this will scan everything into the prologue */
 		*found = parser_scan_multipart_prologue (parser, multipart);
@@ -1917,7 +1968,7 @@ parser_construct_part (GMimeParser *parser, GMimeParserOptions *options)
 	/* get the headers */
 	priv->state = GMIME_PARSER_STATE_HEADERS;
 	while (priv->state < GMIME_PARSER_STATE_HEADERS_END) {
-		if (parser_step (parser) == GMIME_PARSER_STATE_ERROR)
+		if (parser_step (parser, options) == GMIME_PARSER_STATE_ERROR)
 			return NULL;
 	}
 	
@@ -1960,7 +2011,6 @@ parser_construct_message (GMimeParser *parser, GMimeParserOptions *options)
 	ContentType *content_type;
 	GMimeMessage *message;
 	GMimeObject *object;
-	GMimeStream *stream;
 	BoundaryType found;
 	const char *inptr;
 	Header *header;
@@ -1969,18 +2019,19 @@ parser_construct_message (GMimeParser *parser, GMimeParserOptions *options)
 	
 	/* scan the from-line if we are parsing an mbox */
 	while (priv->state != GMIME_PARSER_STATE_MESSAGE_HEADERS) {
-		if (parser_step (parser) == GMIME_PARSER_STATE_ERROR)
+		if (parser_step (parser, options) == GMIME_PARSER_STATE_ERROR)
 			return NULL;
 	}
 	
 	/* parse the headers */
 	while (priv->state < GMIME_PARSER_STATE_HEADERS_END) {
-		if (parser_step (parser) == GMIME_PARSER_STATE_ERROR)
+		if (parser_step (parser, options) == GMIME_PARSER_STATE_ERROR)
 			return NULL;
 	}
 	
 	message = g_mime_message_new (FALSE);
 	((GMimeObject *) message)->ensure_newline = FALSE;
+	_g_mime_header_list_set_options (((GMimeObject *) message)->headers, options);
 	
 	for (i = 0; i < priv->headers->len; i++) {
 		header = priv->headers->pdata[i];
@@ -2018,6 +2069,9 @@ parser_construct_message (GMimeParser *parser, GMimeParserOptions *options)
 	content_type_destroy (content_type);
 	message->mime_part = object;
 	
+	if (priv->state == GMIME_PARSER_STATE_ERROR)
+		_g_mime_parser_options_warn (options, -1, GMIME_WARN_MALFORMED_MESSAGE, NULL);
+	
 	if (priv->format == GMIME_FORMAT_MBOX) {
 		priv->state = GMIME_PARSER_STATE_FROM;
 		parser_pop_boundary (parser);
