@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2014 Jeffrey Stedfast
+ *  Copyright (C) 2000-2017 Jeffrey Stedfast
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public License
@@ -29,6 +29,7 @@
 #include <gmime/gmime-object.h>
 #include <gmime/gmime-message.h>
 #include <gmime/gmime-content-type.h>
+#include <gmime/gmime-parser-options.h>
 #include <gmime/gmime-stream.h>
 
 G_BEGIN_DECLS
@@ -44,6 +45,21 @@ typedef struct _GMimeParser GMimeParser;
 typedef struct _GMimeParserClass GMimeParserClass;
 
 
+/**
+ * GMimeFormat:
+ * @GMIME_FORMAT_MESSAGE: The stream contains a single message.
+ * @GMIME_FORMAT_MBOX: The stream is in the UNIX mbox format.
+ * @GMIME_FORMAT_MMDF: The stream is in the MMDF format.
+ *
+ * An enum of formats.
+ **/
+typedef enum {
+	GMIME_FORMAT_MESSAGE,
+	GMIME_FORMAT_MBOX,
+	GMIME_FORMAT_MMDF
+} GMimeFormat;
+
+
 /**
  * GMimeParser:
  * @parent_object: parent #GObject
@@ -89,8 +105,8 @@ void g_mime_parser_init_with_stream (GMimeParser *parser, GMimeStream *stream);
 gboolean g_mime_parser_get_persist_stream (GMimeParser *parser);
 void g_mime_parser_set_persist_stream (GMimeParser *parser, gboolean persist);
 
-gboolean g_mime_parser_get_scan_from (GMimeParser *parser);
-void g_mime_parser_set_scan_from (GMimeParser *parser, gboolean scan_from);
+GMimeFormat g_mime_parser_get_format (GMimeParser *parser);
+void g_mime_parser_set_format (GMimeParser *parser, GMimeFormat format);
 
 gboolean g_mime_parser_get_respect_content_length (GMimeParser *parser);
 void g_mime_parser_set_respect_content_length (GMimeParser *parser, gboolean respect_content_length);
@@ -99,17 +115,16 @@ void g_mime_parser_set_header_regex (GMimeParser *parser, const char *regex,
 				     GMimeParserHeaderRegexFunc header_cb,
 				     gpointer user_data);
 
-GMimeObject *g_mime_parser_construct_part (GMimeParser *parser);
-
-GMimeMessage *g_mime_parser_construct_message (GMimeParser *parser);
+GMimeObject *g_mime_parser_construct_part (GMimeParser *parser, GMimeParserOptions *options);
+GMimeMessage *g_mime_parser_construct_message (GMimeParser *parser, GMimeParserOptions *options);
 
 gint64 g_mime_parser_tell (GMimeParser *parser);
 
 gboolean g_mime_parser_eos (GMimeParser *parser);
 
-char *g_mime_parser_get_from (GMimeParser *parser);
+char *g_mime_parser_get_mbox_marker (GMimeParser *parser);
+gint64 g_mime_parser_get_mbox_marker_offset (GMimeParser *parser);
 
-gint64 g_mime_parser_get_from_offset (GMimeParser *parser);
 gint64 g_mime_parser_get_headers_begin (GMimeParser *parser);
 gint64 g_mime_parser_get_headers_end (GMimeParser *parser);
 
