@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2014 Jeffrey Stedfast
+ *  Copyright (C) 2000-2017 Jeffrey Stedfast
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public License
@@ -44,6 +44,9 @@
  **/
 
 
+G_DEFINE_BOXED_TYPE (GMimePartIter, g_mime_part_iter, g_mime_part_iter_clone, g_mime_part_iter_free);
+
+
 typedef struct _GMimeObjectStack GMimeObjectStack;
 
 struct _GMimeObjectStack {
@@ -146,6 +149,32 @@ g_mime_part_iter_free (GMimePartIter *iter)
 }
 
 
+/**
+ * g_mime_part_iter_clone:
+ * @iter: a #GMimePartIter
+ *
+ * Clones the @iter, including its current state.
+ *
+ * Returns: (transfer full): a new #GMimePartIter that is identical to @iter.
+ **/
+GMimePartIter *
+g_mime_part_iter_clone (GMimePartIter *iter)
+{
+	GMimePartIter *clone;
+	char *path;
+	
+	g_return_val_if_fail (iter != NULL, NULL);
+	
+	clone = g_mime_part_iter_new (iter->toplevel);
+	if ((path = g_mime_part_iter_get_path (iter))) {
+		g_mime_part_iter_jump_to (clone, path);
+		g_free (path);
+	}
+	
+	return clone;
+}
+
+
 /**
  * g_mime_part_iter_reset:
  * @iter: a #GMimePartIter
