@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2013 Jeffrey Stedfast
+ *  Copyright (C) 2000-2017 Jeffrey Stedfast
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public License
@@ -26,6 +26,8 @@
 
 G_BEGIN_DECLS
 
+#define GMIME_TYPE_PART_ITER (g_mime_part_iter_get_type ())
+
 /**
  * GMimePartIter:
  *
@@ -33,9 +35,13 @@ G_BEGIN_DECLS
  **/
 typedef struct _GMimePartIter GMimePartIter;
 
+GType g_mime_part_iter_get_type (void) G_GNUC_CONST;
+
 GMimePartIter *g_mime_part_iter_new (GMimeObject *toplevel);
 void g_mime_part_iter_free (GMimePartIter *iter);
 
+GMimePartIter *g_mime_part_iter_clone (GMimePartIter *iter);
+
 void g_mime_part_iter_reset (GMimePartIter *iter);
 
 gboolean g_mime_part_iter_jump_to (GMimePartIter *iter, const char *path);
