@@ -130,6 +130,7 @@ g_mime_part_init (GMimePart *mime_part, GMimePartClass *klass)
 	mime_part->content_location = NULL;
 	mime_part->content_md5 = NULL;
 	mime_part->content = NULL;
+	mime_part->openpgp = (GMimeOpenPGPData) -1;
 }
 
 static void
@@ -432,18 +433,6 @@ mime_part_encode (GMimeObject *object, GMimeEncodingConstraint constraint)
 	GMimeFilter *filter;
 	
 	switch (part->encoding) {
-	case GMIME_CONTENT_ENCODING_DEFAULT:
-		/* Unspecified encoding, we need to figure out the
-		 * best encoding no matter what */
-		break;
-	case GMIME_CONTENT_ENCODING_7BIT:
-		/* This encoding is always safe. */
-		return;
-	case GMIME_CONTENT_ENCODING_8BIT:
-		/* This encoding is safe unless the constraint is 7bit. */
-		if (constraint != GMIME_ENCODING_CONSTRAINT_7BIT)
-			return;
-		break;
 	case GMIME_CONTENT_ENCODING_BINARY:
 		/* This encoding is only safe if the constraint is binary. */
 		if (constraint == GMIME_ENCODING_CONSTRAINT_BINARY)
@@ -454,6 +443,8 @@ mime_part_encode (GMimeObject *object, GMimeEncodingConstraint constraint)
 	case GMIME_CONTENT_ENCODING_UUENCODE:
 		/* These encodings are always safe. */
 		return;
+	default:
+		break;
 	}
 	
 	filter = g_mime_filter_best_new (GMIME_FILTER_BEST_ENCODING);
@@ -467,7 +458,26 @@ mime_part_encode (GMimeObject *object, GMimeEncodingConstraint constraint)
 	g_object_unref (stream);
 	
 	encoding = g_mime_filter_best_encoding ((GMimeFilterBest *) filter, constraint);
-	g_mime_part_set_content_encoding (part, encoding);
+	
+	switch (part->encoding) {
+	case GMIME_CONTENT_ENCODING_DEFAULT:
+		g_mime_part_set_content_encoding (part, encoding);
+		break;
+	case GMIME_CONTENT_ENCODING_7BIT:
+		/* This encoding is generally safe, but we may need to encode From-lines. */
+		if (((GMimeFilterBest *) filter)->hadfrom)
+			g_mime_part_set_content_encoding (part, GMIME_CONTENT_ENCODING_QUOTEDPRINTABLE);
+		break;
+	case GMIME_CONTENT_ENCODING_8BIT:
+		if (constraint == GMIME_ENCODING_CONSTRAINT_7BIT)
+			g_mime_part_set_content_encoding (part, encoding);
+		else if (((GMimeFilterBest *) filter)->hadfrom)
+			g_mime_part_set_content_encoding (part, GMIME_CONTENT_ENCODING_QUOTEDPRINTABLE);
+		break;
+	default:
+		break;
+	}
+	
 	g_object_unref (filter);
 }
 
@@ -747,7 +757,7 @@ g_mime_part_set_content_location (GMimePart *mime_part, const char *content_loca
 
 	_g_mime_object_block_header_list_changed (object);
 	g_mime_header_list_set (object->headers, "Content-Location", content_location, NULL);
-	_g_mime_object_block_header_list_changed (object);
+	_g_mime_object_unblock_header_list_changed (object);
 }
 
 
@@ -788,8 +798,11 @@ g_mime_part_set_content_encoding (GMimePart *mime_part, GMimeContentEncoding enc
 	mime_part->encoding = encoding;
 	
 	_g_mime_object_block_header_list_changed (object);
-	g_mime_header_list_set (object->headers, "Content-Transfer-Encoding", value, NULL);
-	_g_mime_object_block_header_list_changed (object);
+	if (value != NULL)
+		g_mime_header_list_set (object->headers, "Content-Transfer-Encoding", value, NULL);
+	else
+		g_mime_header_list_remove (object->headers, "Content-Transfer-Encoding");
+	_g_mime_object_unblock_header_list_changed (object);
 }
 
 
@@ -926,6 +939,8 @@ set_content (GMimePart *mime_part, GMimeDataWrapper *content)
 	if (mime_part->content)
 		g_object_unref (mime_part->content);
 	
+	mime_part->openpgp = (GMimeOpenPGPData) -1;
+	
 	mime_part->content = content;
 	g_object_ref (content);
 }
@@ -1000,6 +1015,27 @@ g_mime_part_get_openpgp_data (GMimePart *mime_part)
 {
 	g_return_val_if_fail (GMIME_IS_PART (mime_part), GMIME_OPENPGP_DATA_NONE);
 	
+	if (mime_part->content == NULL)
+		return GMIME_OPENPGP_DATA_NONE;
+	
+	if (mime_part->openpgp == (GMimeOpenPGPData) -1) {
+		GMimeStream *filtered, *stream;
+		GMimeFilterOpenPGP *openpgp;
+		
+		stream = g_mime_stream_null_new ();
+		filtered = g_mime_stream_filter_new (stream);
+		g_object_unref (stream);
+		
+		openpgp = (GMimeFilterOpenPGP *) g_mime_filter_openpgp_new ();
+		g_mime_stream_filter_add ((GMimeStreamFilter *) filtered, (GMimeFilter *) openpgp);
+		g_mime_data_wrapper_write_to_stream (mime_part->content, filtered);
+		g_mime_stream_flush (filtered);
+		g_object_unref (filtered);
+		
+		mime_part->openpgp = g_mime_filter_openpgp_get_data_type (openpgp);
+		g_object_unref (openpgp);
+	}
+	
 	return mime_part->openpgp;
 }
 
