@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2013 Jeffrey Stedfast
+ *  Copyright (C) 2000-2017 Jeffrey Stedfast
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public License
@@ -32,6 +32,7 @@
 #include <errno.h>
 
 #include "gmime-stream-file.h"
+#include "gmime-error.h"
 
 
 /**
@@ -124,10 +125,9 @@ g_mime_stream_file_init (GMimeStreamFile *stream, GMimeStreamFileClass *klass)
 static void
 g_mime_stream_file_finalize (GObject *object)
 {
-	GMimeStreamFile *stream = (GMimeStreamFile *) object;
+	GMimeStream *stream = (GMimeStream *) object;
 	
-	if (stream->owner && stream->fp)
-		fclose (stream->fp);
+	stream_close (stream);
 	
 	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
@@ -153,7 +153,8 @@ stream_read (GMimeStream *stream, char *buf, size_t len)
 		len = (size_t) MIN (stream->bound_end - stream->position, (gint64) len);
 	
 	/* make sure we are at the right position */
-	fseek (fstream->fp, (long) stream->position, SEEK_SET);
+	if ((fseek (fstream->fp, (long) stream->position, SEEK_SET)) == -1)
+		return -1;
 	
 	if ((nread = fread (buf, 1, len, fstream->fp)) > 0)
 		stream->position += nread;
@@ -181,7 +182,8 @@ stream_write (GMimeStream *stream, const char *buf, size_t len)
 		len = (size_t) MIN (stream->bound_end - stream->position, (gint64) len);
 	
 	/* make sure we are at the right position */
-	fseek (fstream->fp, (long) stream->position, SEEK_SET);
+	if (fseek (fstream->fp, (long) stream->position, SEEK_SET) == -1)
+		return -1;
 
 	if ((nwritten = fwrite (buf, 1, len, fstream->fp)) > 0)
 		stream->position += nwritten;
@@ -206,12 +208,12 @@ static int
 stream_close (GMimeStream *stream)
 {
 	GMimeStreamFile *fstream = (GMimeStreamFile *) stream;
-	int rv;
+	int rv = 0;
 	
 	if (fstream->fp == NULL)
 		return 0;
 	
-	if ((rv = fclose (fstream->fp)) != 0)
+	if (!fstream->owner || (rv = fclose (fstream->fp)) == 0)
 		fstream->fp = NULL;
 	
 	return rv;
@@ -327,9 +329,13 @@ stream_length (GMimeStream *stream)
 	if (stream->bound_start != -1 && stream->bound_end != -1)
 		return stream->bound_end - stream->bound_start;
 	
-	fseek (fstream->fp, (long) 0, SEEK_END);
+	if (fseek (fstream->fp, (long) 0, SEEK_END) == -1)
+		return -1;
+	
 	bound_end = ftell (fstream->fp);
-	fseek (fstream->fp, (long) stream->position, SEEK_SET);
+	
+	if (fseek (fstream->fp, (long) stream->position, SEEK_SET) == -1)
+		return -1;
 	
 	if (bound_end < stream->bound_start) {
 		errno = EINVAL;
@@ -344,12 +350,12 @@ stream_substream (GMimeStream *stream, gint64 start, gint64 end)
 {
 	GMimeStreamFile *fstream;
 	
-	fstream = g_object_newv (GMIME_TYPE_STREAM_FILE, 0, NULL);
-	g_mime_stream_construct (GMIME_STREAM (fstream), start, end);
+	fstream = g_object_new (GMIME_TYPE_STREAM_FILE, NULL);
+	g_mime_stream_construct ((GMimeStream *) fstream, start, end);
+	fstream->fp = ((GMimeStreamFile *) stream)->fp;
 	fstream->owner = FALSE;
-	fstream->fp = GMIME_STREAM_FILE (stream)->fp;
 	
-	return GMIME_STREAM (fstream);
+	return (GMimeStream *) fstream;
 }
 
 
@@ -367,7 +373,6 @@ stream_substream (GMimeStream *stream, gint64 start, gint64 end)
 GMimeStream *
 g_mime_stream_file_new (FILE *fp)
 {
-	GMimeStreamFile *fstream;
 	gint64 start;
 	
 	g_return_val_if_fail (fp != NULL, NULL);
@@ -379,12 +384,7 @@ g_mime_stream_file_new (FILE *fp)
 	if ((start = ftell (fp)) == -1)
 		start = 0;
 	
-	fstream = g_object_newv (GMIME_TYPE_STREAM_FILE, 0, NULL);
-	g_mime_stream_construct (GMIME_STREAM (fstream), start, -1);
-	fstream->owner = TRUE;
-	fstream->fp = fp;
-	
-	return GMIME_STREAM (fstream);
+	return g_mime_stream_file_new_with_bounds (fp, start, -1);
 }
 
 
@@ -413,37 +413,38 @@ g_mime_stream_file_new_with_bounds (FILE *fp, gint64 start, gint64 end)
 	_setmode (_fileno (fp), O_BINARY);
 #endif
 	
-	fstream = g_object_newv (GMIME_TYPE_STREAM_FILE, 0, NULL);
-	g_mime_stream_construct (GMIME_STREAM (fstream), start, end);
+	fstream = g_object_new (GMIME_TYPE_STREAM_FILE, NULL);
+	g_mime_stream_construct ((GMimeStream *) fstream, start, end);
 	fstream->owner = TRUE;
 	fstream->fp = fp;
 	
-	return GMIME_STREAM (fstream);
+	return (GMimeStream *) fstream;
 }
 
 
 /**
- * g_mime_stream_file_new_for_path:
+ * g_mime_stream_file_open:
  * @path: the path to a file
  * @mode: as in fopen(3)
+ * @err: a #GError
  *
  * Creates a new #GMimeStreamFile object for the specified @path.
  *
- * Returns: a stream using for reading and/or writing to the specified
- * file path or %NULL on error.
- *
- * Since: 2.6.18
+ * Returns: (transfer full): a stream using for reading and/or writing to the
+ * specified file path or %NULL on error.
  **/
 GMimeStream *
-g_mime_stream_file_new_for_path (const char *path, const char *mode)
+g_mime_stream_file_open (const char *path, const char *mode, GError **err)
 {
 	FILE *fp;
 	
 	g_return_val_if_fail (path != NULL, NULL);
 	g_return_val_if_fail (mode != NULL, NULL);
 	
-	if (!(fp = fopen (path, mode)))
+	if (!(fp = fopen (path, mode))) {
+		g_set_error (err, GMIME_ERROR, errno, "Failed to open `%s': %s", path, g_strerror (errno));
 		return NULL;
+	}
 	
 	return g_mime_stream_file_new (fp);
 }
