@@ -164,7 +164,7 @@ struct _ah_inject_test {
 	const char *inner_after;
 };
 
-const char * local_recipients[] =
+static const char * local_recipients[] =
 	{
 		"0x0D211DC5D9F4567271AC0582D8DECFBFC9346CD4",
 		NULL,
@@ -729,7 +729,7 @@ test_ah_injection (void)
 					}
 				}
 				
-				encrypted = g_mime_multipart_encrypted_encrypt (ctx, mainpart, TRUE, NULL,
+				encrypted = g_mime_multipart_encrypted_encrypt (ctx, mainpart, FALSE, NULL,
 										GMIME_ENCRYPT_ALWAYS_TRUST,
 										recip, &err);
 				if (!encrypted) {
@@ -1047,11 +1047,11 @@ import_secret_key (void)
 int main (int argc, char **argv)
 {
 	char *gpg;
-
+	
 	g_mime_init ();
 	
 	testsuite_init (argc, argv);
-
+	
 	if (!(gpg = g_find_program_in_path ("gpg2")))
 		if (!(gpg = g_find_program_in_path ("gpg")))
 			return EXIT_FAILURE;
@@ -1062,7 +1062,7 @@ int main (int argc, char **argv)
 	testsuite_start ("Autocrypt: generate headers");
 	test_ah_generation ();
 	testsuite_end ();
-
+	
 #ifdef ENABLE_CRYPTO
 	testsuite_start ("Autocrypt: import OpenPGP secret key");
 	import_secret_key ();
@@ -1076,7 +1076,7 @@ int main (int argc, char **argv)
 	testsuite_start ("Autocrypt: inject headers");
 	test_ah_injection ();
 	testsuite_end ();
-
+	
 	g_mime_shutdown ();
 	
 #ifdef ENABLE_CRYPTO
