@@ -329,12 +329,118 @@ import_key (GMimeCryptoContext *ctx, const char *path)
 	}
 }
 
+static void
+pump_data_through_filter (GMimeFilter *filter, const char *path, GMimeStream *ostream)
+{
+	GMimeStream *onebyte, *filtered, *stream;
+	GMimeFilter *unix2dos, *dos2unix;
+	
+	filtered = g_mime_stream_filter_new (ostream);
+	
+	/* convert to DOS format before piping through the OpenPGP filter to maximize testing area */
+	unix2dos = g_mime_filter_unix2dos_new (FALSE);
+	g_mime_stream_filter_add ((GMimeStreamFilter *) filtered, unix2dos);
+	g_object_unref (unix2dos);
+	
+	g_mime_stream_filter_add ((GMimeStreamFilter *) filtered, filter);
+	
+	/* convert back to UNIX format after filtering */
+	dos2unix = g_mime_filter_dos2unix_new (FALSE);
+	g_mime_stream_filter_add ((GMimeStreamFilter *) filtered, dos2unix);
+	g_object_unref (dos2unix);
+	
+	onebyte = test_stream_onebyte_new (filtered);
+	g_object_unref (filtered);
+	
+	stream = g_mime_stream_fs_open (path, O_RDONLY, 0644, NULL);
+	g_mime_stream_write_to_stream (stream, onebyte);
+	g_mime_stream_flush (onebyte);
+	g_object_unref (onebyte);
+	g_object_unref (stream);
+}
+
+static char *openpgp_data_types[] = {
+	"GMIME_OPENPGP_DATA_NONE",
+	"GMIME_OPENPGP_DATA_ENCRYPTED",
+	"GMIME_OPENPGP_DATA_SIGNED",
+	"GMIME_OPENPGP_DATA_PUBLIC_KEY",
+	"GMIME_OPENPGP_DATA_PRIVATE_KEY"
+};
+
+static void
+test_openpgp_filter (GMimeFilterOpenPGP *filter, const char *path, GMimeOpenPGPData data_type, gint64 begin, gint64 end)
+{
+	GMimeStream *filtered, *stream, *expected, *ostream;
+	GMimeFilter *dos2unix;
+	GMimeOpenPGPData type;
+	Exception *ex = NULL;
+	GByteArray *buf[2];
+	char *filename;
+	struct stat st;
+	gint64 offset;
+	
+	ostream = g_mime_stream_mem_new ();
+	
+	pump_data_through_filter ((GMimeFilter *) filter, path, ostream);
+	
+	if ((type = g_mime_filter_openpgp_get_data_type (filter)) != data_type) {
+		g_object_unref (ostream);
+		throw (exception_new ("Incorrect OpenPGP data type detected: %s", openpgp_data_types[type]));
+	}
+	
+	if ((offset = g_mime_filter_openpgp_get_begin_offset (filter)) != begin) {
+		g_object_unref (ostream);
+		throw (exception_new ("Incorrect begin offset: %ld", (long) offset));
+	}
+	
+	if ((offset = g_mime_filter_openpgp_get_end_offset (filter)) != end) {
+		g_object_unref (ostream);
+		throw (exception_new ("Incorrect end offset: %ld", (long) offset));
+	}
+	
+	filename = g_strdup_printf ("%s.openpgp-block", path);
+	if (stat (filename, &st) == -1) {
+		stream = g_mime_stream_fs_open (filename, O_RDWR | O_CREAT, 0644, NULL);
+		g_mime_stream_reset (ostream);
+		g_mime_stream_write_to_stream (ostream, stream);
+		g_mime_stream_flush (stream);
+		g_mime_stream_reset (stream);
+	} else {
+		stream = g_mime_stream_fs_open (filename, O_RDONLY, 0644, NULL);
+	}
+	g_free (filename);
+	
+	/* make sure the data is in UNIX format before comparing (might be running tests on Windows) */
+	expected = g_mime_stream_mem_new ();
+	filtered = g_mime_stream_filter_new (expected);
+	dos2unix = g_mime_filter_dos2unix_new (FALSE);
+	g_mime_stream_filter_add ((GMimeStreamFilter *) filtered, dos2unix);
+	g_object_unref (dos2unix);
+	
+	g_mime_stream_write_to_stream (stream, filtered);
+	g_mime_stream_flush (filtered);
+	g_object_unref (filtered);
+	g_object_unref (stream);
+	
+	buf[0] = GMIME_STREAM_MEM (expected)->buffer;
+	buf[1] = GMIME_STREAM_MEM (ostream)->buffer;
+	
+	if (buf[0]->len != buf[1]->len || memcmp (buf[0]->data, buf[1]->data, buf[0]->len) != 0)
+		ex = exception_new ("filtered data does not match the expected result");
+	
+	g_object_unref (expected);
+	g_object_unref (ostream);
+	
+	if (ex != NULL)
+		throw (ex);
+}
 
 int main (int argc, char **argv)
 {
 #ifdef ENABLE_CRYPTO
 	const char *datadir = "data/pgp";
 	GMimeStream *istream, *ostream;
+	GMimeFilterOpenPGP *filter;
 	GMimeCryptoContext *ctx;
 	const char *what;
 	char *gpg, *key;
@@ -403,13 +509,13 @@ int main (int argc, char **argv)
 	g_mime_stream_reset (istream);
 	
 	what = "GMimeGpgContext::sign";
-	testsuite_check (what);
+	testsuite_check ("%s", what);
 	try {
 		test_sign (ctx, FALSE, istream, ostream);
 		testsuite_check_passed ();
 		
 		what = "GMimeGpgContext::verify";
-		testsuite_check (what);
+		testsuite_check ("%s", what);
 		g_mime_stream_reset (istream);
 		g_mime_stream_reset (ostream);
 		test_verify (ctx, istream, ostream);
@@ -423,13 +529,13 @@ int main (int argc, char **argv)
 	ostream = g_mime_stream_mem_new ();
 	
 	what = "GMimeGpgContext::sign (detached)";
-	testsuite_check (what);
+	testsuite_check ("%s", what);
 	try {
 		test_sign (ctx, TRUE, istream, ostream);
 		testsuite_check_passed ();
 		
 		what = "GMimeGpgContext::verify (detached)";
-		testsuite_check (what);
+		testsuite_check ("%s", what);
 		g_mime_stream_reset (istream);
 		g_mime_stream_reset (ostream);
 		test_verify_detached (ctx, istream, ostream);
@@ -443,13 +549,13 @@ int main (int argc, char **argv)
 	ostream = g_mime_stream_mem_new ();
 	
 	what = "GMimeGpgContext::encrypt";
-	testsuite_check (what);
+	testsuite_check ("%s", what);
 	try {
 		test_encrypt (ctx, FALSE, istream, ostream);
 		testsuite_check_passed ();
 		
 		what = "GMimeGpgContext::decrypt";
-		testsuite_check (what);
+		testsuite_check ("%s", what);
 		g_mime_stream_reset (istream);
 		g_mime_stream_reset (ostream);
 		test_decrypt (ctx, FALSE, istream, ostream);
@@ -463,13 +569,13 @@ int main (int argc, char **argv)
 	ostream = g_mime_stream_mem_new ();
 	
 	what = "GMimeGpgContext::encrypt+sign";
-	testsuite_check (what);
+	testsuite_check ("%s", what);
 	try {
 		test_encrypt (ctx, TRUE, istream, ostream);
 		testsuite_check_passed ();
 		
 		what = "GMimeGpgContext::decrypt+verify";
-		testsuite_check (what);
+		testsuite_check ("%s", what);
 		g_mime_stream_reset (istream);
 		g_mime_stream_reset (ostream);
 		test_decrypt (ctx, TRUE, istream, ostream);
@@ -481,6 +587,64 @@ int main (int argc, char **argv)
 	g_object_unref (istream);
 	g_object_unref (ostream);
 	g_object_unref (ctx);
+
+	filter = (GMimeFilterOpenPGP *) g_mime_filter_openpgp_new ();
+	
+	what = "GMimeFilterOpenPGP::public key block";
+	testsuite_check ("%s", what);
+	try {
+		key = g_build_filename (datadir, "gmime.gpg.pub", NULL);
+		test_openpgp_filter (filter, key, GMIME_OPENPGP_DATA_PUBLIC_KEY, 0, 1720);
+		g_free (key);
+		
+		testsuite_check_passed ();
+	} catch (ex) {
+		testsuite_check_failed ("%s failed: %s", what, ex->message);
+	} finally;
+	
+	g_mime_filter_reset ((GMimeFilter *) filter);
+	
+	what = "GMimeFilterOpenPGP::private key block";
+	testsuite_check ("%s", what);
+	try {
+		key = g_build_filename (datadir, "gmime.gpg.sec", NULL);
+		test_openpgp_filter (filter, key, GMIME_OPENPGP_DATA_PRIVATE_KEY, 0, 1928);
+		g_free (key);
+		
+		testsuite_check_passed ();
+	} catch (ex) {
+		testsuite_check_failed ("%s failed: %s", what, ex->message);
+	} finally;
+	
+	g_mime_filter_reset ((GMimeFilter *) filter);
+	
+	what = "GMimeFilterOpenPGP::signed message block";
+	testsuite_check ("%s", what);
+	try {
+		key = g_build_filename (datadir, "signed-message.txt", NULL);
+		test_openpgp_filter (filter, key, GMIME_OPENPGP_DATA_SIGNED, 162, 440);
+		g_free (key);
+		
+		testsuite_check_passed ();
+	} catch (ex) {
+		testsuite_check_failed ("%s failed: %s", what, ex->message);
+	} finally;
+	
+	g_mime_filter_reset ((GMimeFilter *) filter);
+	
+	what = "GMimeFilterOpenPGP::encrypted message block";
+	testsuite_check ("%s", what);
+	try {
+		key = g_build_filename (datadir, "encrypted-message.txt", NULL);
+		test_openpgp_filter (filter, key, GMIME_OPENPGP_DATA_ENCRYPTED, 165, 1084);
+		g_free (key);
+		
+		testsuite_check_passed ();
+	} catch (ex) {
+		testsuite_check_failed ("%s failed: %s", what, ex->message);
+	} finally;
+	
+	g_object_unref (filter);
 	
 	testsuite_end ();
 	
